<?php
namespace Drupal\twofactor\EventSubscriber;

use Drupal\user\Entity\User;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\user\UserData;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Routing\RouteMatchInterface;

class CheckTwofactorSubscriber implements EventSubscriberInterface {

  protected $tempStore;
  protected $userData;
  protected $currentUser;
  protected $currentRouteMatch;

  public function __construct(UserData $user_data, AccountProxyInterface $account, RouteMatchInterface $current_route_match) {
    $this->userData = $user_data;
    $this->currentUser = $account;
    $this->currentRouteMatch = $current_route_match;
  }

  /**
   * {@inheritdoc}
   */
  static function getSubscribedEvents() {
    $events[KernelEvents::REQUEST][] = array('checkTwofactor');
    return $events;
  }

  public function checkTwofactor($event) {
    $baseUrl = $event->getRequest()->getBaseUrl();

    // Check if user is logged in.
    if ($this->currentUser->id() > 0) {
      $uid = $this->currentUser->id();
      $session = $event->getRequest()->getSession();

      // Check if twofactor is enabled.
      if ($this->userData->get('twofactor', $uid, 'status') == TRUE) {

        // Check if user has autorized twofactor for this login-session.
        if ($session->get('twofactor.allowed') == 0) {

          $redirect = TRUE;

          // On controller or user logout we dont need a redirect.
          if ($this->currentRouteMatch->getRouteName() == 'twofactor.user_auth') {
            $redirect = FALSE;
          }
          if ($this->currentRouteMatch->getRouteName() == 'user.logout') {
            $redirect = FALSE;
          }
          // Check if Whitelisted IP Adresses are set and yours
          $ip = str_replace(' ', '', $this->userData->get('twofactor', $uid, 'ip'));
          $ip = explode(',', $ip);
          if (in_array($_SERVER['REMOTE_ADDR'], $ip)) {
            $redirect = FALSE;
          }

          if ($redirect == TRUE) {
            $event->setResponse(new RedirectResponse($baseUrl . '/user/' . $uid . '/twofactor/auth'));
          }
        }
        else {
          // Save interaction time.
          $session->set('twofactor.interaction', time());
        }


      }
    }
  }
}
